#! /usr/bin/env python
# -*- coding: utf-8 -*-

"""
Module that contains DCC Selector implementation for Plot Twist
"""

from __future__ import print_function, division, absolute_import

__author__ = "Tomas Poveda"
__license__ = "MIT"
__maintainer__ = "Tomas Poveda"
__email__ = "tpovedatd@gmail.com"

import plottwist
from artellalauncher.core import dccselector


class PlotTwistDCCSelector(dccselector.DCCSelector, object):
    def __init__(self, launcher, parent=None):
        super(PlotTwistDCCSelector, self).__init__(launcher=launcher, parent=parent)

    def ui(self):
        super(PlotTwistDCCSelector, self).ui()

        selector_logo = plottwist.resource.pixmap(name='launcher_logo', extension='png')
        self.add_logo(selector_logo, 930, 0)

        self.set_info_url('https://tpoveda.github.io/plottwist/pipeline/launcher/usage/')

    def _get_title_pixmap(self):
        return plottwist.resource.pixmap(name='title_background', extension='png')


#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Initialization for Plot Twist Tools
"""

from __future__ import print_function, division, absolute_import

__author__ = "Tomas Poveda"
__license__ = "MIT"
__maintainer__ = "Tomas Poveda"
__email__ = "tpovedatd@gmail.com"

print('='*100)
print('| Plot Twist Pipeline | > Loading Plot Twist Tools')

try:
    import plottwist as pt
    from maya import cmds
    cmds.evalDeferred('pt.init()')
    print('| Plot Twist Pipeline | Plot Twist Tools loaded successfully!')
    print('='*100)
except Exception as e:
    try:
        import plottwist as pt
        pt.init()
        print('| Plot Twist Pipeline | Plot Twist Tools loaded successfully!')
        print('='*100)
    except Exception as e:
        print('ERROR: Impossible to load Plot Twist Tools, contact TD!')
        print(str(e))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Module that contains implementation for Plot Twist Shelf
"""

from __future__ import print_function, division, absolute_import

__author__ = "Tomas Poveda"
__license__ = "MIT"
__maintainer__ = "Tomas Poveda"
__email__ = "tpovedatd@gmail.com"

import tpDccLib

import artellapipe


class PlotTwistShelf(tpDccLib.Shelf, object):

    ICONS_PATHS = artellapipe.resource.RESOURCES_FOLDER

    def __init__(self, name='PlotTwistShelf', label_background=(0, 0, 0, 0), label_color=(0.9, 0.9, 0.9), category_icon=None):
        super(PlotTwistShelf, self).__init__(name=name, label_background=label_background, label_color=label_color, category_icon=category_icon)




#! /usr/bin/env python
# -*- coding: utf-8 -*-

"""
Module that contains definitions for character assets in Plot Twist
"""

from __future__ import print_function, division, absolute_import

__author__ = "Tomas Poveda"
__license__ = "MIT"
__maintainer__ = "Tomas Poveda"
__email__ = "tpovedatd@gmail.com"

import artellapipe

from plottwist.core import defines, asset


class PlotTwistCharacterAsset(asset.PlotTwistAsset, object):

    ASSET_TYPE = defines.PLOTTWIST_CHARACTERS_ASSETS
    ASSET_FILES = {
        defines.PLOTTWIST_TEXTURES_ASSET_TYPE: artellapipe.resource.icon(defines.PLOTTWIST_TEXTURES_ASSET_TYPE),
        defines.PLOTTWIST_MODEL_ASSET_TYPE: artellapipe.resource.icon(defines.PLOTTWIST_MODEL_ASSET_TYPE),
        defines.PLOTTWIST_SHADING_ASSET_TYPE: artellapipe.resource.icon(defines.PLOTTWIST_SHADING_ASSET_TYPE),
        defines.PLOTTWIST_RIG_ASSET_TYPE: artellapipe.resource.icon(defines.PLOTTWIST_RIG_ASSET_TYPE),
        defines.PLOTTWIST_GROOM_ASSET_TYPE: artellapipe.resource.icon(defines.PLOTTWIST_GROOM_ASSET_TYPE)
    }

    def __init__(self, project, asset_data):
        super(PlotTwistCharacterAsset, self).__init__(project=project, asset_data=asset_data)



#! /usr/bin/env python
# -*- coding: utf-8 -*-

"""
Module that contains implementations for the different asset files supported by Plot Twist
"""

from __future__ import print_function, division, absolute_import

__author__ = "Tomas Poveda"
__license__ = "MIT"
__maintainer__ = "Tomas Poveda"
__email__ = "tpovedatd@gmail.com"

import os

from tpPyUtils import path as path_utils

import tpDccLib as tp

from artellapipe.core import assetfile, artellalib, defines as artella_defines
from artellapipe.tools.alembicmanager import alembicmanager

from plottwist.core import defines


class TexturesAssetFile(assetfile.ArtellaAssetType, object):

    FILE_TYPE = defines.PLOTTWIST_TEXTURES_ASSET_TYPE
    FILE_EXTENSIONS = defines.PLOTTWIST_TEXTURE_EXTENSIONS

    def __init__(self, asset):
        super(TexturesAssetFile, self).__init__(asset=asset)

    def _get_working_server_versions(self, working_path, artella_data, force_update=False):
        """
        Overrides base ArtellaAssetType _get_working_server_versions function
        Internal function that returns all the working server versions of the current file type in the wrapped asset
        :param working_path: str, working path in Artella server
        :param artella_data: ArtellaReferencesMetaData
        :param force_update: bool, Whether to update Artella data or not (this can take time)
        :return:
        """

        if not artella_data:
            return

        ref_path = os.path.join(working_path, artella_data.name)
        file_name = os.path.basename(ref_path)
        for extension in self.FILE_EXTENSIONS:
            if file_name.endswith(extension):
                if not force_update and self._working_history:
                    return self._working_history
                else:
                    return artellalib.get_asset_history(ref_path)


class ModelAssetFile(assetfile.ArtellaAssetType, object):

    FILE_TYPE = defines.PLOTTWIST_MODEL_ASSET_TYPE
    FILE_EXTENSIONS = [defines.PLOTTWIST_MODEL_EXTENSION]

    def __init__(self, asset):
        super(ModelAssetFile, self).__init__(asset=asset)


class ShadingAssetFile(assetfile.ArtellaAssetType, object):

    FILE_TYPE = defines.PLOTTWIST_SHADING_ASSET_TYPE
    FILE_EXTENSIONS = [defines.PLOTTWIST_SHADING_EXTENSION]

    def __init__(self, asset):
        super(ShadingAssetFile, self).__init__(asset=asset)

    def _get_working_path(self, asset_name, asset_path):
        """
        Overrides base ArtellaAssetType _get_working_path function
        Internal function that returns working path of the current asset file
        :param asset_name: str
        :param asset_path: str
        :return: str
        """

        return path_utils.clean_path(os.path.join(asset_path, artella_defines.ARTELLA_WORKING_FOLDER, self.FILE_TYPE, asset_name+'_SHD'+self.FILE_EXTENSIONS[0]))

    def _get_published_path(self, asset_name, asset_path, version_folder):
        """
        Overrides base ArtellaAssetType _get_published_path function
        Internal function that returns published path of the current asset file
        :param asset_name: str
        :param asset_path: str
        :param version_folder: str
        :return: str
        """

        return path_utils.clean_path(os.path.join(asset_path, version_folder, self.FILE_TYPE, asset_name+'_SHD'+self.FILE_EXTENSIONS[0]))

    def _get_working_server_versions(self, working_path, artella_data, force_update=False):
        """
        Overrides base ArtellaAssetType _get_working_server_versions function
        Internal function that returns all the working server versions of the current file type in the wrapped asset
        :param working_path: str, working path in Artella server
        :param artella_data: ArtellaReferencesMetaData
        :param force_update: bool, Whether to update Artella data or not (this can take time)
        :return:
        """

        if not artella_data:
            return

        asset_name = self._asset.get_name()
        ref_path = os.path.join(working_path, artella_data.name)
        file_name = os.path.basename(ref_path)
        for extension in self.FILE_EXTENSIONS:
            if file_name == '{}_SHD{}'.format(asset_name, extension) or file_name == '{}_shd{}'.format(asset_name, extension):
                if not force_update and self._working_history:
                    return self._working_history
                else:
                    return artellalib.get_asset_history(ref_path)


class RigAssetFile(assetfile.ArtellaAssetType, object):

    FILE_TYPE = defines.PLOTTWIST_RIG_ASSET_TYPE
    FILE_EXTENSIONS = [defines.PLOTTWIST_RIG_EXTENSION]

    def __init__(self, asset):
        super(RigAssetFile, self).__init__(asset=asset)

    def _get_working_path(self, asset_name, asset_path):
        """
        Overrides base ArtellaAssetType _get_working_path function
        Internal function that returns working path of the current asset file
        :param asset_name: str
        :param asset_path: str
        :return: str
        """

        return path_utils.clean_path(os.path.join(asset_path, artella_defines.ARTELLA_WORKING_FOLDER, self.FILE_TYPE, asset_name+'_RIG'+self.FILE_EXTENSIONS[0]))

    def _get_published_path(self, asset_name, asset_path, version_folder):
        """
        Overrides base ArtellaAssetType _get_published_path function
        Internal function that returns published path of the current asset file
        :param asset_name: str
        :param asset_path: str
        :param version_folder: str
        :return: str
        """

        return path_utils.clean_path(os.path.join(asset_path, version_folder, self.FILE_TYPE, asset_name+'_RIG'+self.FILE_EXTENSIONS[0]))


class GroomAssetFile(assetfile.ArtellaAssetType, object):

    FILE_TYPE = defines.PLOTTWIST_GROOM_ASSET_TYPE

    def __init__(self, asset):
        super(GroomAssetFile, self).__init__(asset=asset)


class AlembicAssetFile(assetfile.ArtellaAssetType, object):

    FILE_TYPE = defines.PLOTTWIST_MODEL_ASSET_TYPE
    FILE_EXTENSIONS = [defines.PLOTTWIST_ALEMBIC_EXTENSION]

    def _import_file(self, path, fix_path=True):
        """
        Internal function that imports current file in DCC
        :param fix_path: bool
        :param path: str
        """

        alembicmanager.importer.import_alembic(project=self.get_project(), alembic_path=path, fix_path=fix_path)

    def _reference_file(self, path, fix_path=True):
        """
        Internal function that references current file in DCC
        :param fix_path: bool
        :param path: str
        """

        if tp.is_houdini():
            alembicmanager.importer.import_alembic(project=self.get_project(), alembic_path=path, fix_path=fix_path)
        else:
            alembicmanager.importer.reference_alembic(project=self.get_project(), alembic_path=path, fix_path=fix_path)



#! /usr/bin/env python
# -*- coding: utf-8 -*-

"""
Module that contains implementation for Plot Twist Artella tray
"""

from __future__ import print_function, division, absolute_import

__author__ = "Tomas Poveda"
__license__ = "MIT"
__maintainer__ = "Tomas Poveda"
__email__ = "tpovedatd@gmail.com"

import artellapipe
from artellapipe.gui import tray

from Qt.QtWidgets import *


class PlotTwistTray(tray.ArtellaTray, object):
    def __init__(self, project, parent=None):
        super(PlotTwistTray, self).__init__(project=project, parent=parent)

    def create_menu(self):
        menu = QMenu(self)

        project_name = self._project.name.title()

        documentation_icon = artellapipe.plottwist.resource.icon('documentation')
        project_icon = artellapipe.plottwist.resource.icon('plottwist', theme=None)
        artella_icon = artellapipe.plottwist.resource.icon('artella')
        email_icon = artellapipe.plottwist.resource.icon('message')
        kitsu_icon = artellapipe.plottwist.resource.icon('kitsu')
        drive_icon = artellapipe.plottwist.resource.icon('drive')

        doc_action = QAction(documentation_icon, '{} Documentation'.format(project_name), self, statusTip='Open {} Tools Documentation webpage'.format(project_name), triggered=self._on_open_documentation)
        web_action = QAction(project_icon, '{} Web'.format(project_name), self, statusTip='Open {} Project webpage'.format(project_name), triggered=self._on_open_webpage)
        artella_action = QAction(artella_icon, '{} Artella Project'.format(project_name), self, statusTip='Open {} Artella Project webpage'.format(project_name), triggered=self._on_open_artella_project)
        email_action = QAction(email_icon, 'Send Email', self, statusTip='Send Email to {} TD team'.format(project_name), triggered=self._on_send_email)
        kitsu_action = QAction(kitsu_icon, '{} Kitsu webpage'.format(project_name), self, statusTip='Open {} Kitsu webpage'.format(project_name), triggered=self._on_open_kitsu)
        drive_action = QAction(drive_icon, '{} Drive webpage'.format(project_name), self, statusTip='Open {} Drive webpage'.format(project_name), triggered=self._on_open_drive)
        for action in [doc_action, web_action, artella_action, email_action, kitsu_action, drive_action]:
            menu.addAction(action)

        return menu

    def _on_open_documentation(self):
        self.project.open_documentation()

    def _on_open_webpage(self):
        self.project.open_webpage()

    def _on_open_artella_project(self):
        self.project.open_artella_project_url()

    def _on_send_email(self):
        self.project.send_email()

    def _on_open_kitsu(self):
        self.project.open_kitsu()

    def _on_open_drive(self):
        self.project.open_drive()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Module that contains widget implementation for asset viewer for Plot Twist
"""

from __future__ import print_function, division, absolute_import

__author__ = "Tomas Poveda"
__license__ = "MIT"
__maintainer__ = "Tomas Poveda"
__email__ = "tpovedatd@gmail.com"

import artellapipe
from artellapipe.core import defines, assetsviewer

from plottwist.core import asset


class PlotTwistAssetsViewer(assetsviewer.AssetsViewer, object):

    ASSET_WIDGET_CLASS = asset.PlotTwistAssetWidget

    def __init__(self, project, column_count=3, parent=None):
        super(PlotTwistAssetsViewer, self).__init__(project=project, column_count=column_count, parent=parent)

    def change_category(self, category=None):
        """
        Overrides base AssetsViewer change_category function
        :param category: str
        """

        if not category:
            category = defines.ARTELLA_ALL_CATEGORIES_NAME

        categories_dict = dict()
        asset_types = self._project.update_types_from_kitsu(force=False)
        for asset_type in asset_types:
            categories_dict[asset_type] = asset_type

        if category != defines.ARTELLA_ALL_CATEGORIES_NAME and category not in categories_dict.keys():
            artellapipe.logger.warning('Asset Type {} is not a valid asset type for project {}'.format(category, self._project.name.title()))
            category = defines.ARTELLA_ALL_CATEGORIES_NAME
        else:
            if category != defines.ARTELLA_ALL_CATEGORIES_NAME:
                category = categories_dict[category]

        self.clear()

        new_assets = list()
        for new_asset in reversed(self._assets):
            if category == defines.ARTELLA_ALL_CATEGORIES_NAME:
                new_asset.setVisible(True)
                new_assets.insert(0, new_asset)
            else:
                if new_asset.asset.get_category().name == category:
                    new_asset.setVisible(True)
                    new_assets.insert(0, new_asset)
                else:
                    new_asset.setVisible(False)
                    new_assets.append(new_asset)

        for new_asset in new_assets:
            self._add_widget(new_asset)




#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Tool that allows to detect errors and trace calls for Plot Twist
"""

from __future__ import print_function, division, absolute_import

__author__ = "Tomas Poveda"
__license__ = "MIT"
__maintainer__ = "Tomas Poveda"
__email__ = "tpovedatd@gmail.com"


import artellapipe
from artellapipe.tools.bugtracker import bugtracker


class PlotTwistBugTracker(bugtracker.ArtellaBugTracker, object):
    def __init__(self, project, tool_name=None, parent=None):
        super(PlotTwistBugTracker, self).__init__(project=project, tool_name=tool_name, parent=parent)

    def ui(self):
        super(PlotTwistBugTracker, self).ui()

        logo_pixmap = artellapipe.plottwist.resource.pixmap(name='bugtracker_logo', extension='png')
        self.set_logo(logo_pixmap)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Module that contains implementation for Kitsu Login Form
"""

from __future__ import print_function, division, absolute_import

__author__ = "Tomas Poveda"
__license__ = "MIT"
__maintainer__ = "Tomas Poveda"
__email__ = "tpovedatd@gmail.com"

from Qt.QtCore import *
from Qt.QtWidgets import *

from tpQtLib.core import qtutils
from tpQtLib.widgets import formwidget, lightbox, stack

import artellapipe
from artellapipe.gui import dialog
from artellapipe.utils import worker

from plottwist.pipeline.kitsu import loginwidget


class KitsuLoginForm(formwidget.FormDialog, object):

    loginAccepted = Signal(str, str, bool)
    loginCancelled = Signal()

    def __init__(self, email='', password='', store_credentials=False, parent=None):
        super(KitsuLoginForm, self).__init__(parent=parent)

        self._valid_fields = False

        self.setMinimumWidth(300)
        self.setMinimumHeight(300)
        self.setMaximumWidth(400)
        self.setMaximumHeight(400)
        self.accept_button().setText('Login')
        self.accept_button().setEnabled(False)

        kitsu_pixmap = artellapipe.plottwist.resource.pixmap('kitsu', category='icons', theme='color')
        kitsu_icon = QLabel()
        kitsu_icon.setAlignment(Qt.AlignCenter)
        kitsu_icon.setPixmap(kitsu_pixmap)
        self.main_layout.insertWidget(2, kitsu_icon)

        login_form = {
            'title': 'Kitsu Login',
            'description': 'Write your Kitsu credentials',
            'layout': 'vertical',
            'schema': [
                {
                    'name': 'email',
                    'type': 'string',
                    'value': email
                },
                {
                    'name': 'password',
                    'type': 'password',
                    'value': password
                },
                {
                    'name': 'store credentials',
                    'type': 'bool',
                    'value': bool(store_credentials)
                }
            ],
            'validator': self._kitsu_login_validator,
            'accepted': self._kitsu_login_accepted,
            'rejected': self._kitsu_login_cancelled
        }
        self.set_settings(login_form)
        self._form_widget.validate()

    def _kitsu_login_validator(self, **kwargs):
        """
        Internal function that is called each time the user updates the form
        :param kwargs:
        """

        username = kwargs.get('email', None)
        password = kwargs.get('password', None)

        if not username or not password:
            self._valid_fields = False
            self.accept_button().setEnabled(False)
        else:
            self._valid_fields = True
            self.accept_button().setEnabled(True)

    def _kitsu_login_accepted(self, **kwargs):
        """
        Internal callback function that is called when the user presses the login button
        :param kwargs: dict
        """

        username = kwargs.get('email', None)
        password = kwargs.get('password', None)
        store = kwargs.get('store credentials', False)

        self.loginAccepted.emit(username, password, store)

    def _kitsu_login_cancelled(self, **kwargs):
        """
        Internal callback function that is called when the user presses the cancel button
        :param kwargs: dict
        """

        self.loginCancelled.emit()

    def _on_validated(self):
        """
        Overrides base _on_validated FormWidget function
        """

        self._accept_btn.setEnabled(not self._form_widget.has_errors() and self._valid_fields)


class KitsuLoginDialog(dialog.ArtellaDialog, object):

    validLogin = Signal()
    invalidLogin = Signal()

    def __init__(self, project, parent=None):

        self._project = project

        super(KitsuLoginDialog, self).__init__(
            name='KitsuLoginDialog',
            title='Kitsu - Login Dialog',
            parent=parent,
            size=(400, 400),
            fixed_size=True
        )

        self._kitsu_worker = worker.Worker(app=QApplication.instance())
        self._kitsu_worker.workCompleted.connect(self._on_kitsu_worker_completed)
        self._kitsu_worker.workFailure.connect(self._on_kitsu_worker_failure)
        self._kitsu_worker.start()

    def ui(self):
        super(KitsuLoginDialog, self).ui()

        self._main_title._button_closed.setVisible(False)
        self._main_title.setVisible(False)

        self._main_stack = stack.SlidingStackedWidget(parent=self)

        self._login_widget = self._create_login_form()
        self._kitsu_waiter = loginwidget.KitsuLoginWidget()

        self._main_stack.addWidget(self._login_widget)
        self._main_stack.addWidget(self._kitsu_waiter)

        self._lightbox = lightbox.Lightbox(self)
        self._lightbox.set_widget(self._main_stack)
        self._lightbox.show()

        self._lightbox.closed.connect(self.close)

    def setup_signals(self):
        self._main_stack.animFinished.connect(self._on_stack_anim_finished)
        self._login_widget.loginAccepted.connect(self._on_kitsu_login_accepted)
        self._login_widget.loginCancelled.connect(self.close)

    def _create_login_form(self):
        """
        Internal callback function that checks current Kitsu login status
        """

        kitsu_user = self._project.kitsu_user
        kitsu_password = self._project.kitsu_password
        store_credentials = self._project.kitsu_store_credentials

        return KitsuLoginForm(email=kitsu_user, password=kitsu_password, store_credentials=store_credentials)

    def _start_kitsu_worker(self, new_index=None):
        """
        Internal function that starts new Kitsu worker
        :param new_index: int
        """

        if new_index:
            w = self._main_stack.widget(new_index)
        else:
            w = self._main_stack.currentWidget()
        if w == self._kitsu_waiter:
            self._kitsu_worker.queue_work(self._kitsu_login, {})

    def _kitsu_login(self, data):
        """
        Internal function that is called by Kitsu Worker to execute login
        :param data:
        :return:
        """

        if not self._kitsu_user or not self._kitsu_password:
            artellapipe.plottwist.logger.warning('Impossible to login into Kitsu because user and password are not given!')
            return

        valid_login = artellapipe.plottwist.login_kitsu(self._kitsu_user, self._kitsu_password)

        return valid_login

    def _on_stack_anim_finished(self, new_index):
        """
        Internal callback function that is callded when stack anim finished
        :param new_index: int
        """

        self._start_kitsu_worker(new_index=new_index)

    def _on_kitsu_login_accepted(self, email, password, store_credentials):
        """
        Internal callback function that is called when the user successfully log into Kitsu through Kitsu login form
        :param email: variant, str or None
        :param password: variant, str or None
        :param store_credentials: bool
        """

        self._kitsu_user = email
        self._kitsu_password = password
        self._kitsu_store = store_credentials

        index = self._main_stack.indexOf(self._kitsu_waiter)
        self._main_stack.slide_in_index(index, force=True)

    def _on_kitsu_worker_completed(self, uid, valid_login):
        """
        Internal callback function that is called when Kitsu worker finishes
        :param uid: str
        :param valid_login: bool
        """

        if valid_login:
            if not self._project.is_logged_in_kitsu():
                artellapipe.plottwist.logger.warning('Something when wrong during Kitsu login')
                return False
            if self._kitsu_user and self._kitsu_password:
                self._project.settings.set('kitsu_store_credentials', bool(self._kitsu_store))
                self.validLogin.emit()
                self.close()
        else:
            qtutils.show_error(self, 'Error while logging into Kitsu', 'Kitsu credentials are not valid. Try again please!')
            self._main_stack.slide_in_index(0)
            return False

        return True

    def _on_kitsu_worker_failure(self, uid, msg):
        """
        Internal callback function that is called when Kitsu worker fails
        :param uid: str
        :param msg: str
        """

        # self._kitsu_worker.stop()
        self._kitsu_user = None
        self._kitsu_password = None
        self._kitsu_store = False
        artellapipe.logger.error('{} | {}'.format(uid, msg))
        self._main_stack.slide_in_index(0)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Module that contains widget implementation that shows user info for Plot Twist
"""

from __future__ import print_function, division, absolute_import

__author__ = "Tomas Poveda"
__license__ = "MIT"
__maintainer__ = "Tomas Poveda"
__email__ = "tpovedatd@gmail.com"

from Qt.QtCore import *
from Qt.QtWidgets import *
from Qt.QtGui import *

from tpQtLib.core import qtutils

import artellapipe
from artellapipe.core import userinfo

from plottwist.pipeline.kitsu import logindialog, userballoon


class PlotTwistUserInfo(userinfo.UserInfo, object):

    login = Signal()
    logout = Signal()

    def __init__(self, project, parent=None):
        super(PlotTwistUserInfo, self).__init__(
            project=project if project else artellapipe.plottwist,
            parent=parent
        )

        self.main_layout.addSpacerItem(QSpacerItem(10, 0, QSizePolicy.Expanding, QSizePolicy.Preferred))

        shutdown_icon = artellapipe.plottwist.resource.icon('shutdown')
        self._kitsu_off_icon = artellapipe.plottwist.resource.icon('kitsu_off')
        self._kitsu_on_icon = artellapipe.plottwist.resource.icon('kitsu_on')

        self._kitsu_btn = QPushButton('Kitsu')
        self._kitsu_btn.setIconSize(QSize(50, 50))
        self._kitsu_btn.setFixedHeight(25)
        self._kitsu_btn.setMinimumWidth(90)
        self._kitsu_btn.setFlat(True)

        self._kitsu_logout_btn = QPushButton()
        self._kitsu_logout_btn.setVisible(False)
        self._kitsu_logout_btn.setIcon(shutdown_icon)
        self._kitsu_btn.setFlat(True)

        self.main_layout.addWidget(self._kitsu_btn)
        self.main_layout.addWidget(self._kitsu_logout_btn)

        self._kitsu_btn.clicked.connect(self._on_open_kitsu_login)
        self._kitsu_logout_btn.clicked.connect(self._on_kitsu_logout)

        self.update_kitsu_status()

    def try_kitsu_login(self):
        """
        Function that tries to log into Kitsu
        """

        valid_login = self._project.login_kitsu()
        if valid_login:
            self._kitsu_login()
            return True

        return False

    def update_kitsu_status(self):
        """
        Synchronizes current Kitsu status between UserInfo and current project
        """

        if not self._project:
            artellapipe.plottwist.logger.warning('Impossible to update Kitsu Status because Project is not defined!')
            return

        if self._project.is_logged_in_kitsu():
            self._kitsu_btn.setIcon(self._kitsu_on_icon)
            self._kitsu_logout_btn.setVisible(True)
        else:
            self._kitsu_btn.setIcon(self._kitsu_off_icon)
            self._kitsu_logout_btn.setVisible(False)

    def _kitsu_login(self):
        """
        Tries to login into Kitsu
        """

        self.update_kitsu_status()
        self.login.emit()

    def _on_open_kitsu_login(self):
        """
        Internal callback function that is called when the user presses the Kitsu button
        """

        if artellapipe.plottwist.is_logged_in_kitsu():
            self._ballon = userballoon.KitsuUserBalloon()
            rect_btn = self._kitsu_btn.geometry()
            rect_balloon = self._ballon.geometry()

            pos = QCursor.pos()
            pos.setX(pos.x() - (self._kitsu_btn.width() / 2) - 20)

            rect_balloon.setRect(
                pos.x(), pos.y(), rect_btn.width(), rect_btn.height()
            )

            self._ballon.setGeometry(rect_balloon)
            self._ballon.show()

        else:
            login_dialog = logindialog.KitsuLoginDialog(project=self._project)
            login_dialog.validLogin.connect(self._on_kitsu_login)
            # login_dialog.invalidLogin.connect(self._on_kitsu_logout)
            login_dialog.exec_()

    def _on_kitsu_login(self):
        """
        Internal callback function that is claled when the user presses the login button
        """

        self._kitsu_login()

    def _on_kitsu_logout(self):
        """
        Internal callback function that is called when the user presses the logout button
        """

        remove_credentials = False
        res = qtutils.show_question(self, 'Kitsu Logout', 'Do you want to remove Kitsu stored credentials?')
        if res == QMessageBox.Yes:
            remove_credentials = True

        valid = artellapipe.plottwist.logout_kitsu(remove_credentials=remove_credentials)
        if not valid:
            artellapipe.plottwist.logger.warning('Error while logging out from Kitsu')
            artellapipe.plottwist.logger.warning('Something when wrong during Kitsu login out')
            return

        self.update_kitsu_status()
        self.logout.emit()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Module that contains implementation for background workers
"""

from __future__ import print_function, division, absolute_import

__author__ = "Tomas Poveda"
__license__ = "MIT"
__maintainer__ = "Tomas Poveda"
__email__ = "tpovedatd@gmail.com"

from Qt.QtCore import *
from Qt.QtGui import *

import os

import artellapipe

from plottwist.pipeline.kitsu import kitsulib


class KitsuImageDownloaderWorker(QRunnable, object):

    class KitsuImageDownloaderWorkerSignals(QObject, object):
        triggered = Signal(object)

    def __init__(self, *args):
        super(KitsuImageDownloaderWorker, self).__init__(*args)

        self._path = None
        self._preview_id = None
        self.signals = KitsuImageDownloaderWorker.KitsuImageDownloaderWorkerSignals()

    def set_preview_id(self, preview_id):
        """
        Sets the preview ID linked to Kitsu project
        :param preview_id: str
        """

        self._preview_id = preview_id

    def set_path(self, path):
        """
        Sets the path where image should be downloaded
        :param path: str
        """

        self._path = path

    def run(self):
        try:
            if self._preview_id and self._path:
                if not os.path.isfile(self._path):
                    kitsulib.download_preview_file_thumbnail(self._preview_id, self._path)
                if not os.path.isfile(self._path):
                    icon = None
                else:
                    icon_pixmap = QPixmap(self._path)
                    icon = QIcon(icon_pixmap)
                self.signals.triggered.emit(icon)
        except Exception as e:
            artellapipe.plottwist.logger.error('Cannot load thumbnail image: {}!'.format(e))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Module that contains widget implementation for assets manager widget for Plot Twist
"""

from __future__ import print_function, division, absolute_import

__author__ = "Tomas Poveda"
__license__ = "MIT"
__maintainer__ = "Tomas Poveda"
__email__ = "tpovedatd@gmail.com"

from artellapipe.tools.assetsmanager.widgets import assetswidget


class PlotTwistAssetsWidget(assetswidget.AssetsWidget, object):
    def __init__(self, project, parent=None):
        super(PlotTwistAssetsWidget, self).__init__(project=project, parent=parent)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Module that contains implementation for Kitsu User Balloon
"""

from __future__ import print_function, division, absolute_import

__author__ = "Tomas Poveda"
__license__ = "MIT"
__maintainer__ = "Tomas Poveda"
__email__ = "tpovedatd@gmail.com"

from Qt.QtWidgets import *

from tpQtLib.widgets import balloon

import artellapipe


class KitsuUserBalloon(balloon.BalloonDialog, object):
    def __init__(self, parent=None):
        super(KitsuUserBalloon, self).__init__(parent)

        main_layout = QVBoxLayout()
        self.setLayout(main_layout)

        user_info = artellapipe.plottwist.kitsu_user_data()

        full_name_lbl = QLabel('{} - {}'.format(user_info.full_name, user_info.role))
        email_lbl = QLabel(user_info.email)
        timezone_lbl = QLabel('{} - {}'.format(user_info.locale, user_info.timezone))

        main_layout.addWidget(full_name_lbl)
        main_layout.addWidget(email_lbl)
        main_layout.addWidget(timezone_lbl)








#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Tool that allow to manage scene assets for Plot Twist
"""

from __future__ import print_function, division, absolute_import

__author__ = "Tomas Poveda"
__license__ = "MIT"
__maintainer__ = "Tomas Poveda"
__email__ = "tpovedatd@gmail.com"

import tpDccLib as tp

import artellapipe
from artellapipe.gui import window
from artellapipe.tools.outliner import outliner


class PlotTwistOutlinerWidget(outliner.ArtellaOutlinerWidget, object):

    title = 'Plot Twist - Outliner'

    def __init__(self, project, parent=None):
        super(PlotTwistOutlinerWidget, self).__init__(project=project, parent=parent)


class PlotTwistOutliner(outliner.ArtellaOutliner, object):
    def __init__(self, project, parent=None):
        super(PlotTwistOutliner, self).__init__(project=project, parent=parent)


def run():
    if tp.is_maya():
        win = window.dock_window(project=artellapipe.plottwist, window_class=PlotTwistOutlinerWidget)
        return win
    else:
        win = PlotTwistOutliner(project=artellapipe.plottwist)
        win.show()
        return win



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Tool that allow artists to interact with Artella functionality inside DCCS in Plot Twist
"""

from __future__ import print_function, division, absolute_import

__author__ = "Tomas Poveda"
__license__ = "MIT"
__maintainer__ = "Tomas Poveda"
__email__ = "tpovedatd@gmail.com"

import os
from functools import partial

from Qt.QtCore import *
from Qt.QtWidgets import *

import tpDccLib as tp

from tpQtLib.widgets import stack

import artellapipe
from artellapipe.gui import window, button
from artellapipe.tools.assetslibrary import assetslibrary

from plottwist.core import userinfo


class PlotTwistAssetsLibraryWidget(assetslibrary.ArtellaAssetsLibraryWidget, object):

    name = 'PlotTwistAssetsLibrary'
    title = 'Plot Twist - Assets Viewer'

    def __init__(self, project, parent=None):
        super(PlotTwistAssetsLibraryWidget, self).__init__(project=project, parent=parent)

        self._try_kitsu_login()

    def ui(self):
        super(PlotTwistAssetsLibraryWidget, self).ui()

        self._user_info = userinfo.PlotTwistUserInfo(project=artellapipe.plottwist)
        self.main_layout.addWidget(self._user_info)

        self._main_stack = stack.SlidingStackedWidget()
        self.main_layout.addWidget(self._main_stack)

        no_kitsu_log_widget = QFrame()
        no_kitsu_log_widget.setFrameShape(QFrame.StyledPanel)
        no_kitsu_log_widget.setFrameShadow(QFrame.Sunken)
        no_kitsu_log_layout = QVBoxLayout()
        no_kitsu_log_layout.setContentsMargins(0, 0, 0, 0)
        no_kitsu_log_layout.setSpacing(0)
        no_kitsu_log_widget.setLayout(no_kitsu_log_layout)
        no_items_lbl = QLabel()
        no_items_pixmap = artellapipe.plottwist.resource.pixmap('kitsu_no_login')
        no_items_lbl.setPixmap(no_items_pixmap)
        no_items_lbl.setAlignment(Qt.AlignCenter)
        no_kitsu_log_layout.addItem(QSpacerItem(0, 10, QSizePolicy.Preferred, QSizePolicy.Expanding))
        no_kitsu_log_layout.addWidget(no_items_lbl)
        no_kitsu_log_layout.addItem(QSpacerItem(0, 10, QSizePolicy.Preferred, QSizePolicy.Expanding))

        self._main_stack.addWidget(no_kitsu_log_widget)
        self._main_stack.addWidget(self._main_widget)

        self._user_info.login.connect(self._on_kitsu_login)
        self._user_info.logout.connect(self._on_kitsu_logout)

    def _try_kitsu_login(self):
        """
        Internal function that tries to login into Kitsu
        """

        valid_login = self._user_info.try_kitsu_login()
        if valid_login:
            self._assets_viewer.update_assets()
            return True

        return False

    def _on_kitsu_login(self):
        self._assets_viewer.update_assets()
        self._main_stack.slide_in_index(1)

    def _on_kitsu_logout(self):
        self._main_stack.slide_in_index(0)
        self._assets_viewer.clear_assets()

        # self._update_assets_status()

    # def _update_assets_status(self):
    #     """
    #     Internal function that checks asset availability an enables sync button if necessary
    #     """
    #
    #     for i in range(self._assets_viewer.rowCount()):
    #         for j in range(self._assets_viewer.columnCount()):
    #             item = self._assets_viewer.cellWidget(i, j)
    #             if not item:
    #                 continue
    #             asset_widget = item.containedWidget
    #             rig_file_type = asset_widget.asset.get_file_type(defines.SOLSTICE_RIG_ASSET_TYPE)
    #             if rig_file_type:
    #                 published_path = rig_file_type.get_latest_local_published_path()
    #                 if published_path and os.path.isfile(published_path):
    #                     continue
    #             self._create_sync_button(item)
    #
    # def _create_sync_button(self, item):
    #     """
    #     Internal function that creates a sync button in the given item
    #     :param item: ArtellaAssetWidget
    #     """
    #
    #     sync_icon = artellapipe.solstice.resource.icon('sync')
    #     sync_hover_icon = artellapipe.solstice.resource.icon('sync_hover')
    #     sync_btn = button.IconButton(icon=sync_icon, icon_hover=sync_hover_icon, icon_min_size=50)
    #     sync_btn.setIconSize(QSize(50, 50))
    #     sync_btn.move(item.width() * 0.5 - sync_btn.width() * 0.5, item.height() * 0.5 - sync_btn.height() * 0.5)
    #     sync_btn.setParent(item.containedWidget)
    #
    #     not_published_pixmap = artellapipe.solstice.resource.pixmap('asset_not_published')
    #     not_published_lbl = QLabel()
    #     not_published_lbl.move(9, 9)
    #     not_published_lbl.setFixedSize(65, 65)
    #     not_published_lbl.setPixmap(not_published_pixmap)
    #     not_published_lbl.setParent(item.containedWidget)
    #
    #     asset_widget = item.containedWidget
    #     sync_btn.clicked.connect(partial(asset_widget.asset.sync_latest_published_files, None, True))


class PlotTwistAssetsLibrary(assetslibrary.ArtellaAssetsLibrary, object):

    LIBRARY_WIDGET = PlotTwistAssetsLibraryWidget

    def __init__(self, project, parent=None):
        super(PlotTwistAssetsLibrary, self).__init__(project=project, parent=parent)


def run():
    if tp.is_maya():
        win = window.dock_window(project=artellapipe.plottwist, window_class=PlotTwistAssetsLibraryWidget)
        return win
    else:
        win = PlotTwistAssetsLibrary(project=artellapipe.plottwist)
        win.show()
        return win



#! /usr/bin/env python
# -*- coding: utf-8 -*-

"""
Module that contains definitions for asset in Plot Twist
"""

from __future__ import print_function, division, absolute_import

__author__ = "Tomas Poveda"
__license__ = "MIT"
__maintainer__ = "Tomas Poveda"
__email__ = "tpovedatd@gmail.com"

import os
import webbrowser

from Qt.QtCore import *
from Qt.QtWidgets import *

import artellapipe
from artellapipe.core import asset as artella_asset

from plottwist.core import defines
from plottwist.pipeline.utils import worker


class PlotTwistAsset(artella_asset.ArtellaAsset, object):
    def __init__(self, project, asset_data):
        super(PlotTwistAsset, self).__init__(project=project, asset_data=asset_data)

    def get_name(self):
        """
        Overrides ArtellaAsset get_name function
        Returns the path of the asset
        :return: str
        """

        return self.data.name

    def get_path(self):
        """
        Overrides ArtellaAsset get_path function
        Returns the path of the asset
        :return: str
        """

        entity_type = self._project.get_entity_type_by_id(self.data.entity_type_id)
        if not entity_type:
            return ''

        entity_type_name = entity_type.name
        entity_path = os.path.join(self._project.get_assets_path(), entity_type_name, self.get_name())

        return entity_path

    def get_category(self):
        """
        Overrides abstract get_thumbnail_icon function
        Returns the category of the asset
        :return: str
        """

        entity_type_id = self.data.entity_type_id
        return self._project.get_entity_type_by_id(entity_type_id)

        return self.data.type

    def get_shading_type(self):
        """
        Implements base ArtellaAsset get_shading_type function
        Returns the asset file type of the shading file for the project
        :return: str
        """

        return defines.PLOTTWIST_SHADING_ASSET_TYPE

    def get_preview_id(self):
        """
        Returns the preview ID of the asset in Kitsu
        :return: str
        """

        return self.data.preview_file_id

    def get_kitsu_assets_url(self):
        """
        Returns Kitsu URL of the assets
        :return: str
        """

        return self._project.get_kitsu_assets_url()

    def open_in_kitsu(self):
        """
        Opens current asset in Kitsu web
        """

        kitsu_asset_url = self.get_kitsu_assets_url()
        asset_url = kitsu_asset_url + '/' + self.data.id
        webbrowser.open(asset_url)

    def _get_file_name(self, asset_name, **kwargs):
        """
        Overrides base ArtellaAsset _get_file_name function
        :param asset_name: str
        :param kwargs: dict
        :return: str
        """

        return self._project.solve_name('asset_file', asset_name)


class PlotTwistAssetWidget(artella_asset.ArtellaAssetWidget, object):

    THUMB_SIZE = (150, 150)

    ThreadPool = QThreadPool()

    def __init__(self, asset, parent=None):

        self._icon_path = None
        self._thumbnail_icon = None
        self._pixmap = None
        self._pixmap_rect = None
        self._pixmap_scaled = None

        super(PlotTwistAssetWidget, self).__init__(asset=asset, parent=parent)

        self._worker = worker.KitsuImageDownloaderWorker()
        self._worker.setAutoDelete(False)
        self._worker.signals.triggered.connect(self._on_thumbnail_from_image)
        self._worker_started = False

        self.asset.get_path()

        self._update_thumbnail_icon()

    def ui(self):
        super(PlotTwistAssetWidget, self).ui()

    def get_thumbnail_path(self):
        """
        Overrides base ArtellaAssetWidget get_thumbnail_path function
        :return: str
        """

        data_path = artellapipe.plottwist.get_data_path()
        thumbnails_cache_folder = os.path.join(data_path, 'asset_thumbs_cache')
        if not os.path.isdir(thumbnails_cache_folder):
            os.makedirs(thumbnails_cache_folder)

        return os.path.join(thumbnails_cache_folder, self.asset.get_name()+'.png')

    def get_thumbnail_icon(self):
        """
        Overrides base ArtellaAssetWidget get_thumbnail_icon function
        :return: QIcon
        """

        return self._thumbnail_icon

    def _create_context_menu(self, context_menu):
        """
        Overrides ArtellaAssetWidget _create_context_menu function
        :param context_menu: QMenu
        """

        super(PlotTwistAssetWidget, self)._create_context_menu(context_menu)

        kitsu_icon = artellapipe.plottwist.resource.icon('kitsu')

        kitsu_action = QAction(kitsu_icon, 'Open in Kitsu', context_menu)

        kitsu_action.triggered.connect(self._on_open_in_kitsu)

        artella_action = context_menu.find_action('Open in Artella')
        if artella_action:
            context_menu.insertAction(artella_action, kitsu_action)
        else:
            context_menu.addAction(kitsu_action)

    def _update_thumbnail_icon(self):
        """
        Internal function that updates the thumbnail icon
        :return:
        """

        thumbnail_path = self.get_thumbnail_path()
        preview_id = self._asset.get_preview_id()
        self._worker_started = True
        self._worker.set_path(thumbnail_path)
        self._worker.set_preview_id(preview_id)
        self.ThreadPool.start(self._worker)
        self._thumbnail_icon = self.DEFAULT_ICON
        self._asset_btn.setIcon(self._thumbnail_icon)

        return self._thumbnail_icon

    def _on_thumbnail_from_image(self, asset_icon):
        """
        Internal callback function that is called when an image object has finished loading
        """

        if asset_icon and not asset_icon.isNull():
            self._thumbnail_icon = asset_icon
            self._asset_btn.setIcon(asset_icon)

    def _on_open_in_kitsu(self):
        """
        Internal callback function that is called when the user presses Open in Kitsu contextual menu action
        """

        self._asset.open_in_kitsu()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Utility module that contains useful utilities and classes related with Kitsu
"""

from __future__ import print_function, division, absolute_import

__author__ = "Tomas Poveda"
__license__ = "MIT"
__maintainer__ = "Tomas Poveda"
__email__ = "tpovedatd@gmail.com"

import gazu

from plottwist.pipeline.kitsu import kitsuclasses


def set_host(new_host):
    """
    Sets current configured host on which requests are sent
    :param new_host: variant, str or None
    """

    gazu.set_host(new_host)


def host_is_up():
    """
    Returns whether host is up or not
    :return: bool
    """

    return gazu.client.host_is_up()


def log_in(email, password):
    """
    Login to Kitsu with given email and password
    :param email: str
    :param password: str
    :return:
    """

    return gazu.log_in(email, password)


def get_current_user(as_dict=False):
    """
    Returns user database information for user linked to auth tokens
    :param as_dict: bool
    :return: variant, dict or KitsuUserInfo
    """

    user_data = gazu.client.get_current_user()
    if as_dict:
        return user_data

    return kitsuclasses.KitsuUserInfo(user_data)


def all_assets_for_project(project_id, as_list=False):
    """
    Returns a list with all assets in the given project
    :param project_id: str
    :param as_list: bool
    :return: variant, dict or list
    """

    if type(project_id) != dict:
        project_id = {'id': project_id}

    assets_list = gazu.asset.all_assets_for_project(project_id)
    if as_list:
        return assets_list

    return [kitsuclasses.KitsuAsset(asset) for asset in assets_list]


def all_asset_types_for_project(project_id, as_dict=False):
    """
    Returns asset types from assets listed in the given project
    :param project_id: variant, str or dict
    :param as_dict: bool, Whether to return data as dict or as Kitsu class
    :return: variant, list(dict) or list(KitsuAssetType)
    """

    if type(project_id) != dict:
        project_id = {'id': project_id}

    asset_types = gazu.asset.all_asset_types_for_project(project_id)
    if as_dict:
        return asset_types

    return [kitsuclasses.KitsuAssetType(asset_type) for asset_type in asset_types]


def download_preview_file_thumbnail(preview_id, file_path):
    """
    Downloads given preview file thumbnail and save it at given location
    :param preview_id:  str or dict, The preview file dict or ID.
    :param target_path: str, Location on ahrd drive where to save the file.
    """

    if type(preview_id) != dict:
        preview_id = {'id': preview_id}

    gazu.files.download_preview_file_thumbnail(preview_id, file_path)


def get_project_entity_types():
    """
    Returns all entity types in the project
    :return: list
    """

    return gazu.client.fetch_one('entity-types', '')



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Module that contains implementation for Kitsu Login widget
"""

from __future__ import print_function, division, absolute_import

__author__ = "Tomas Poveda"
__license__ = "MIT"
__maintainer__ = "Tomas Poveda"
__email__ = "tpovedatd@gmail.com"

from Qt.QtCore import *
from Qt.QtWidgets import *

import artellapipe
from artellapipe.gui import spinner


class KitsuLoginWidget(QFrame, object):
    def __init__(self, parent=None):
        super(KitsuLoginWidget, self).__init__(parent)

        self.setStyleSheet("#background {border-radius: 3px;border-style: solid;border-width: 1px;border-color: rgb(32,32,32);}")
        self.setFrameShape(QFrame.StyledPanel)
        self.setFrameShadow(QFrame.Raised)

        main_layout = QVBoxLayout()
        main_layout.setContentsMargins(0, 0, 0, 0)
        main_layout.setSpacing(0)
        self.setLayout(main_layout)

        kitsu_pixmap = artellapipe.plottwist.resource.pixmap('kitsu', category='icons', theme='color')
        kitsu_icon = QLabel()
        kitsu_icon.setAlignment(Qt.AlignCenter)
        kitsu_icon.setPixmap(kitsu_pixmap)
        main_layout.addWidget(kitsu_icon)

        self.wait_spinner = spinner.WaitSpinner(spinner_type=spinner.SpinnerType.Loading)
        self.wait_spinner._bg.setFrameShape(QFrame.NoFrame)
        self.wait_spinner._bg.setFrameShadow(QFrame.Plain)

        # main_layout.addItem(QSpacerItem(0, 20, QSizePolicy.Fixed, QSizePolicy.Expanding))
        main_layout.addWidget(self.wait_spinner)
        # main_layout.addItem(QSpacerItem(0, 20, QSizePolicy.Fixed, QSizePolicy.Expanding))



#! /usr/bin/env python
# -*- coding: utf-8 -*-

"""
Module that contains implementation for Plot Twist Artella updater
"""

from __future__ import print_function, division, absolute_import

__author__ = "Tomas Poveda"
__license__ = "MIT"
__maintainer__ = "Tomas Poveda"
__email__ = "tpovedatd@gmail.com"

import os

from tpPyUtils import path as path_utils

from artellalauncher.core import defines, updater as core_updater

import plottwist


def get_updater_config_path():
    """
    Returns path where default Artella updater config is located
    :return: str
    """

    return path_utils.clean_path(os.path.join(plottwist.get_project_path(), defines.ARTELLA_UPDATER_CONFIG_FILE_NAME))


class PlotTwistUpdater(core_updater.ArtellaUpdater, object):

    UPDATER_CONFIG_PATH = get_updater_config_path()

    def __init__(self, launcher, project, parent=None):
        super(PlotTwistUpdater, self).__init__(launcher=launcher, project=project, parent=parent)



#! /usr/bin/env python
# -*- coding: utf-8 -*-

"""
Initialization module for plottwist
"""

from __future__ import print_function, division, absolute_import

__author__ = "Tomas Poveda"
__license__ = "MIT"
__maintainer__ = "Tomas Poveda"
__email__ = "tpovedatd@gmail.com"

import os
import inspect

from tpPyUtils import importer, path as path_utils
from tpQtLib.core import resource as resource_utils

import artellapipe as artella
from artellapipe.core import defines


# =================================================================================

logger = None
resource = None

# =================================================================================


class PlotTwistResource(resource_utils.Resource, object):
    RESOURCES_FOLDER = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'resources')


class PlotTwist(importer.Importer, object):
    def __init__(self):
        super(PlotTwist, self).__init__(module_name='plottwist')

    def get_module_path(self):
        """
        Returns path where tpNameIt module is stored
        :return: str
        """

        try:
            mod_dir = os.path.dirname(inspect.getframeinfo(inspect.currentframe()).filename)
        except Exception:
            try:
                mod_dir = os.path.dirname(__file__)
            except Exception:
                try:
                    import tpDccLib
                    mod_dir = tpDccLib.__path__[0]
                except Exception:
                    return None

        return mod_dir


def init(do_reload=False, import_libs=True):
    """
    Initializes module
    :param do_reload: bool, Whether to reload modules or not
    """

    if import_libs:
        import tpPyUtils
        tpPyUtils.init(do_reload=do_reload)
        import tpDccLib
        tpDccLib.init(do_reload=do_reload)
        import tpQtLib
        tpQtLib.init(do_reload=do_reload)
        import tpNameIt
        tpNameIt.init(do_reload=do_reload)
        import artellapipe
        artellapipe.init(do_reload=do_reload)

    from plottwist.core import project

    plottwist_importer = importer.init_importer(importer_class=PlotTwist, do_reload=do_reload)

    plottwist_importer.import_modules()

    global logger
    global resource
    logger = plottwist_importer.logger
    resource = PlotTwistResource

    plottwist_importer.import_modules()
    plottwist_importer.import_packages(
        only_packages=True,
        order=['plottwist.launcher', 'plottwist.core'],
        skip_modules=['plottwist.pipeline.userSetup']
    )
    if do_reload:
        plottwist_importer.reload_all()

    naming_file = path_utils.clean_path(os.path.join(os.path.dirname(inspect.getframeinfo(inspect.currentframe()).filename), 'naming.json'))

    artella.set_project(project.PlotTwist, resource, naming_file)


def get_project_path():
    """
    Returns path where default Artella project is located
    :return: str
    """

    return path_utils.clean_path(os.path.dirname(__file__))


def get_project_config_path():
    """
    Returns path where default Artella project config is located
    :return: str
    """

    return path_utils.clean_path(os.path.join(get_project_path(), defines.ARTELLA_PROJECT_CONFIG_FILE_NAME))


def get_project_changelog_path():
    """
    Returns path where default Artella project changelog is located
    :return: str
    """

    return path_utils.clean_path(os.path.join(get_project_path(), defines.ARTELLA_PROJECT_CHANGELOG_FILE_NAME))


def get_project_shelf_path():
    """
    Returns path where Plot Twist project shelf file is located
    :return: str
    """

    return path_utils.clean_path(os.path.join(get_project_path(), defines.ARTELLA_PROJECT_SHELF_FILE_NAME))


def get_project_menu_path():
    """
    Returns path where Plot Twist project menu file is located
    :return: str
    """

    return path_utils.clean_path(os.path.join(get_project_path(), defines.ARTELLA_PROJECT_SHELF_FILE_NAME))


def get_project_version_path():
    """
    Returns path where version file is located
    :return: str
    """

    return path_utils.clean_path(os.path.join(get_project_path(), defines.ARTELLA_PROJECT_DEFAULT_VERSION_FILE_NAME))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Module that contains all constant definitions used by Plot Twist
"""

from __future__ import print_function, division, absolute_import

__author__ = "Tomas Poveda"
__license__ = "MIT"
__maintainer__ = "Tomas Poveda"
__email__ = "tpovedatd@gmail.com"

from artellapipe.core import defines


# Defines the asset type used for Props
PLOTTWIST_PROP_ASSETS = 'Props'

# Defines the asset type used for Characters
PLOTTWIST_CHARACTERS_ASSETS = 'Characters'

# Defines of textures asset type for Plot Twist
PLOTTWIST_TEXTURES_ASSET_TYPE = defines.ARTELLA_TEXTURES_ASSET_TYPE

# Defines of model asset type for Plot Twist
PLOTTWIST_MODEL_ASSET_TYPE = defines.ARTELLA_MODEL_ASSET_TYPE

# Defines of shading asset type for Plot Twist
PLOTTWIST_SHADING_ASSET_TYPE = defines.ARTELLA_SHADING_ASSET_TYPE

# Defines of rig asset type for Plot Twist
PLOTTWIST_RIG_ASSET_TYPE = defines.ARTELLA_RIG_ASSET_TYPE

# Defines of groom asset type for Plot Twist
PLOTTWIST_GROOM_ASSET_TYPE = defines.ARTELLA_GROOM_ASSET_TYPE

# Defines Alembic extension used by Plot Twist
PLOTTWIST_ALEMBIC_EXTENSION = '.abc'

# Defines Alembic extension used by Plot Twist
PLOTTWIST_MODEL_EXTENSION = '.ma'

# Defines Texture extensions used by Plot Twist
PLOTTWIST_TEXTURE_EXTENSIONS = ['.tx', '.png', '.jpg', '.jpeg']

# Defines Alembic extension used by Plot Twist
PLOTTWIST_SHADING_EXTENSION = '.ma'

# Defines Alembic extension used by Plot Twist
PLOTTWIST_RIG_EXTENSION = '.ma'



import os
import sys
from setuptools import setup

source_path = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'source')
if source_path and os.path.isdir(source_path):
    if source_path not in sys.path:
        sys.path.append(source_path)

from plottwist.__version__ import __version__

setup()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Tool used to manage metadata for Plot Twist
"""

from __future__ import print_function, division, absolute_import

__author__ = "Tomas Poveda"
__license__ = "MIT"
__maintainer__ = "Tomas Poveda"
__email__ = "tpovedatd@gmail.com"

import artellapipe
from artellapipe.tools.tagger import tagger


class PlotTwistTagger(tagger.ArtellaTagger, object):
    def __init__(self, project):
        super(PlotTwistTagger, self).__init__(project=project)


def run():
    win = PlotTwistTagger(artellapipe.plottwist)
    win.show()
    return win



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Tool that allow artists to interact with Artella functionality inside DCCS in Plot Twist
"""

from __future__ import print_function, division, absolute_import

__author__ = "Tomas Poveda"
__license__ = "MIT"
__maintainer__ = "Tomas Poveda"
__email__ = "tpovedatd@gmail.com"

from Qt.QtCore import *
from Qt.QtWidgets import *


import artellapipe
from artellapipe.tools.assetsmanager.core import assetsmanager

from plottwist.core import userinfo
from plottwist.pipeline.tools.assetsmanager.widgets import assetswidget
from plottwist.pipeline.kitsu import loginwidget


class PlotTwistAssetsManager(assetsmanager.ArtellaAssetsManager, object):

    USER_INFO_CLASS = userinfo.PlotTwistUserInfo
    ASSET_WIDGET_CLASS = assetswidget.PlotTwistAssetsWidget

    def __init__(self):
        super(PlotTwistAssetsManager, self).__init__(
            project=artellapipe.plottwist, auto_start_assets_viewer=False
        )

        self._kitsu_store = False
        self._kitsu_user_data = None

        self._menu_bar.setVisible(False)

        self._try_kitsu_login()

    def ui(self):
        super(PlotTwistAssetsManager, self).ui()

        self._kitsu_login_widget = loginwidget.KitsuLoginWidget()

        no_kitsu_log_widget = QFrame()
        no_kitsu_log_widget.setFrameShape(QFrame.StyledPanel)
        no_kitsu_log_widget.setFrameShadow(QFrame.Sunken)
        no_kitsu_log_layout = QVBoxLayout()
        no_kitsu_log_layout.setContentsMargins(0, 0, 0, 0)
        no_kitsu_log_layout.setSpacing(0)
        no_kitsu_log_widget.setLayout(no_kitsu_log_layout)
        no_items_lbl = QLabel()
        no_items_pixmap = artellapipe.plottwist.resource.pixmap('kitsu_no_login')
        no_items_lbl.setPixmap(no_items_pixmap)
        no_items_lbl.setAlignment(Qt.AlignCenter)
        no_kitsu_log_layout.addItem(QSpacerItem(0, 10, QSizePolicy.Preferred, QSizePolicy.Expanding))
        no_kitsu_log_layout.addWidget(no_items_lbl)
        no_kitsu_log_layout.addItem(QSpacerItem(0, 10, QSizePolicy.Preferred, QSizePolicy.Expanding))

        self._main_stack.addWidget(self._kitsu_login_widget)
        self._main_stack.addWidget(no_kitsu_log_widget)

    def setup_signals(self):
        super(PlotTwistAssetsManager, self).setup_signals()

        self._user_info.login.connect(self._on_kitsu_login)
        self._user_info.logout.connect(self._on_kitsu_logout)

    def _setup_menubar(self):
        """
        Overrides base ArtellaManager _setup_menubar function
        """

        menubar_widget = super(PlotTwistAssetsManager, self)._setup_menubar()

        self._project_kitsu_btn = QToolButton(self)
        self._project_kitsu_btn.setText('Kitsu')
        self._project_kitsu_btn.setIcon(artellapipe.plottwist.resource.icon('kitsu'))
        self._project_kitsu_btn.setToolButtonStyle(Qt.ToolButtonTextUnderIcon)
        self._project_kitsu_btn.setVisible(False)
        self._project_kitsu_btn.clicked.connect(self._on_open_project_in_kitsu)

        menubar_widget.layout().addWidget(self._project_kitsu_btn, 0, 3, 1, 1, Qt.AlignCenter)

        return menubar_widget

    def _update_types(self):
        """
        Internal function that updates current asset types with the info retrieve from Kitsu
        """

        self._project.update_entity_types_from_kitsu(force=True)
        asset_types = self._project.update_types_from_kitsu(force=True)
        category_names = [asset_type.name for asset_type in asset_types]
        self._assets_widget.update_asset_categories(category_names)

    def _try_kitsu_login(self):
        """
        Internal function that tries to login into Kitsu
        """

        valid_login = self._user_info.try_kitsu_login()
        if not valid_login:
            self._main_stack.slide_in_index(2)

    def _on_open_project_in_kitsu(self):
        """
        Internal callback function that is called when the user presses on Kitsu button in menu bar
        """

        if not self._project:
            return

        self._project.open_in_kitsu()

    def _on_kitsu_login(self):
        self._update_types()
        self._assets_widget.update_assets()
        self._main_stack.slide_in_index(0)
        self.show_ok_message('Successfully logged into Kitsu')

    def _on_kitsu_logout(self):
        self._main_stack.slide_in_index(2)
        self._assets_widget.clear_assets()
        self.show_ok_message('Successfully logged out from Kitsu')

    # def _kitsu_login(self, data):
    #     """
    #     Internal function that is called by Kitsu Worker to execute login
    #     :param data:
    #     :return:
    #     """
    #
    #     kitsu_user = self._project.kitsu_user
    #     kitsu_password = self._project.kitsu_password
    #
    #     if not kitsu_user or not kitsu_password:
    #         artellapipe.plottwist.logger.warning('Impossible to login into Kitsu because user and password are not given!')
    #         return
    #
    #     valid_login = artellapipe.plottwist.login_kitsu(kitsu_user, kitsu_password)
    #     self._user_info.update_kitsu_status()
    #     if valid_login:
    #         self._update_types()
    #         return True
    #     else:
    #         self._main_stack.slide_in_index(2)
    #         return False
    #
    # def _on_kitsu_login_accepted(self, email, password, store_credentials):
    #     """
    #     Internal callback function that is called when the user successfully log into Kitsu through Kitsu login form
    #     :param email: variant, str or None
    #     :param password: variant, str or None
    #     :param store_credentials: bool
    #     """
    #
    #     self._enable_kitsu(email, password, store_credentials)
    #
    # def _on_kitsu_worker_completed(self, uid, valid_login):
    #     """
    #     Internal callback function that is called when Kitsu worker finishes
    #     :param uid: str
    #     :param valid_login: bool
    #     """
    #
    #     self._main_stack.slide_in_index(0)
    #     self._user_info.setVisible(True)
    #     if valid_login:
    #         if not self._project.is_logged_in_kitsu():
    #             artellapipe.plottwist.logger.warning('Something when wrong during Kitsu login')
    #             return
    #         self.show_ok_message('Successfully logged into Kitsu')
    #         if self._project.kitsu_user and self._project.kitsu_pass:
    #             self.settings().set('kitsu_email', self.kitsu_password)
    #             self.settings().set('kitsu_password', self._kitsu_pass)
    #             self.settings().set('kitsu_store_credentials', bool(self._kitsu_store))
    #         self._project_kitsu_btn.setVisible(True)
    #         self._assets_widget.update_assets()
    #     else:
    #         self._user_info.update_kitsu_status()
    #         self._assets_widget.clear_assets()
    #         self._main_stack.slide_in_index(2, force=True)
    #         self.show_error_message('Error while login into Kitsu. Try again please!')
    #
    # def _on_kitsu_worker_failure(self, uid, msg):
    #     """
    #     Internal callback function that is called when Kitsu worker fails
    #     :param uid: str
    #     :param msg: str
    #     """
    #
    #     self._project.kitsu_user = None
    #     self._project.kitsu_password = None
    #     self._kitsu_store = False
    #     self.show_error_message('{} | {}'.format(uid, msg))
    #     artellapipe.logger.error('{} | {}'.format(uid, msg))
    #     self._main_stack.slide_in_index(0)
    #     self._user_info.setVisible(True)


def run():
    win = PlotTwistAssetsManager()
    win.show()

    return win



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Module that contains implementation for Kitsu data objets to work with a more OO approach
"""

from __future__ import print_function, division, absolute_import

__author__ = "Tomas Poveda"
__license__ = "MIT"
__maintainer__ = "Tomas Poveda"
__email__ = "tpovedatd@gmail.com"


class KitsuUserInfo(object):
    def __init__(self, data):
        super(KitsuUserInfo, self).__init__()

        self._dict = data
        self._first_name = data.get('first_name', '')
        self._last_name = data.get('last_name', '')
        self._has_avatar = data.get('has_avatar', False)
        self._locale = data.get('locale', '')
        self._last_presence = data.get('last_presence', None)
        self._created_at = data.get('created_at', '')
        self._notifications_enabled = data.get('notifications_enabled', None)
        self._shotgun_id = data.get('shotgun_id', None)
        self._updated_at = data.get('updated_at', '')
        self._email = data.get('email', '')
        self._phone = data.get('phone', '')
        self._role = data.get('role', '')
        self._data = data.get('data', None)
        self._full_name = data.get('full_name', '')
        self._timezone = data.get('timezone', '')
        self._active = data.get('active', False)
        self._skills = data.get('skills', list())
        self._desktop_login = data.get('desktop_login', '')
        self._type = data.get('type', '')
        self._id = data.get('id', '')

    @property
    def first_name(self):
        """
        Returns first name of the user
        :return: str
        """

        return self._first_name

    @property
    def last_name(self):
        """
        Returns last name of the user
        :return: str
        """

        return self._last_name

    @property
    def locale(self):
        """
        Returns locale of the user
        :return: str
        """

        return self._locale

    @property
    def full_name(self):
        """
        Returns full name of the user
        :return: str
        """

        return self._full_name

    @property
    def email(self):
        """
        Returns the email of the user
        :return: str
        """

        return self._email

    @property
    def role(self):
        """
        Returns the role of the user
        :return: str
        """

        return self._role

    @property
    def timezone(self):
        """
        Returns time zone of the user
        :return: str
        """

        return self._timezone


class KitsuAsset(object):
    def __init__(self, data):
        super(KitsuAsset, self).__init__()

        self._dict = data

        self._name = data.get('name', '')
        self._id = data.get('id', '')
        self._type = data.get('type', None)
        self._entity_type_id = data.get('entity_type_id', '')
        self._code = data.get('code', None)
        self._description = data.get('description', '')
        self._canceled = data.get('canceled', False)
        self._preview_file_id = data.get('preview_file_id', None)
        self._created_at = data.get('created_at', '')
        self._instance_casting = data.get('instance_casting', list())
        self._shotgun_id = data.get('shotgun_id', None)
        self._updated_at = data.get('updated_at', '')
        self._data = data.get('data', dict())
        self._parent_id = data.get('parent_id', None)
        self._source_id = data.get('source_id', None)
        self._entities_in = data.get('entities_in', list())
        self._project_id = data.get('project_id', '')
        self._entities_out = data.get('entities_out', list())
        self._number_frames = data.get('nb_frames', None)

    @property
    def name(self):
        """
        Returns asset name
        :return: str
        """

        return self._name

    @property
    def id(self):
        """
        Returns asset id
        :return: str
        """

        return self._id

    @property
    def type(self):
        """
        Returns asset type
        :return: str
        """

        return self._type

    @property
    def entity_type_id(self):
        """
        Returns asset entity type id
        :return: str
        """

        return self._entity_type_id

    @property
    def preview_file_id(self):
        """
        Returns asset preview file id
        :return: str
        """

        return self._preview_file_id


class KitsuAssetType(object):
    def __init__(self, data):
        super(KitsuAssetType, self).__init__()

        self._dict = data

        self._name = data.get('name', '')
        self._id = data.get('id', '')
        self._type = data.get('type', '')
        self._created_at = data.get('created_at', '')
        self._updated_at = data.get('updated_at')

    @property
    def name(self):
        """
        Returns asset type name
        :return: str
        """

        return self._name

    @property
    def id(self):
        """
        Returns asset type id
        :return: str
        """

        return self._id

    @property
    def type(self):
        """
        Returns asset Kitsu type
        :return: str
        """

        return self._type

    @property
    def created_at(self):
        """
        Returns data where the asset type was created
        :return: str
        """

        return self._created_at

    @property
    def updated_at(self):
        """
        Returns last date when the asset type was updated
        :return: str
        """

        return self._updated_at


class KitsuEntityType(object):
    def __init__(self, data):
        super(KitsuEntityType, self).__init__()

        self._dict = data

        self._id = data.get('id', '')
        self._name = data.get('name', '')
        self._type = data.get('type', '')
        self._created_at = data.get('created_at', '')
        self._updated_at = data.get('updated_at', '')

    @property
    def id(self):
        """
        Returns entity type id
        :return: str
        """

        return self._id

    @property
    def name(self):
        """
        Returns entity type name
        :return: str
        """

        return self._name

    @property
    def type(self):
        """
        Returns entity type type
        :return: str
        """

        return self._type

    @property
    def created_at(self):
        """
        Returns created date of the entity type
        :return: str
        """

        return self._created_at

    @property
    def updated_at(self):
        """
        Returns last update date of the entity type
        :return: str
        """

        return self._updated_at



#! /usr/bin/env python
# -*- coding: utf-8 -*-

"""
Module that contains implementation for Plot Twist Artella project
"""

from __future__ import print_function, division, absolute_import

__author__ = "Tomas Poveda"
__license__ = "MIT"
__maintainer__ = "Tomas Poveda"
__email__ = "tpovedatd@gmail.com"

import webbrowser
try:
    from urllib.parse import quote
except ImportError:
    from urllib2 import quote

from artellapipe.core import project as artella_project

import plottwist
from plottwist.core import asset, assetsviewer, shelf
from plottwist.launcher import tray
from plottwist.pipeline.kitsu import kitsulib, kitsuclasses


class PlotTwist(artella_project.ArtellaProject):

    PROJECT_PATH = plottwist.get_project_path()
    TRAY_CLASS = tray.PlotTwistTray
    SHELF_CLASS = shelf.PlotTwistShelf
    ASSET_CLASS = asset.PlotTwistAsset
    ASSETS_VIEWER_CLASS = assetsviewer.PlotTwistAssetsViewer
    PROJECT_CONFIG_PATH = plottwist.get_project_config_path()
    PROJECT_CHANGELOG_PATH = plottwist.get_project_changelog_path()
    PROJECT_SHELF_FILE_PATH = plottwist.get_project_shelf_path()
    PROJECT_MENU_FILE_PATH = plottwist.get_project_menu_path()
    PROJECT_VERSION_FILE_PATH = plottwist.get_project_version_path()

    def __init__(self, resource, naming_file):

        self._documentation_url = None
        self._kitsu_id = None
        self._kitsu_url = None
        self._drive_url = None
        self._gazu_api = None

        self._kitsu_user_data = None
        self._kitsu_is_logged = False

        self._entity_types = list()

        super(PlotTwist, self).__init__(resource=resource, naming_file=naming_file)

        self._init_kitsu()

    def init_config(self):
        """
        Overrides base ArtellaProject init_config function to load extra attributes from configuration file
        """

        super(PlotTwist, self).init_config()

        project_config_data = self.get_config_data()
        if not project_config_data:
            return

        self._documentation_url = project_config_data.get('PROJECT_DOCUMENTATION_URL', None)
        self._kitsu_id = project_config_data.get('PROJECT_KITSU_ID', None)
        self._kitsu_url = project_config_data.get('PROJECT_KITSU_URL', None)
        self._drive_url = project_config_data.get('PROJECT_DRIVE_URL', None)
        self._gazu_api = project_config_data.get('GAZU_API', None)

    def find_all_assets(self, asset_name=None):
        """
        Overrides base ArtellaProject find_all_assets function
        :param asset_name: str, If given, a list with the given item will be returned instead
        :return: variant, ArtellaAsset or list(ArtellaAsset)
        """

        if not self.is_logged_in_kitsu():
            self.logger.warning('Impossible to retrieve assets because user is not logged into Kitsu!')
            return

        assets_path = self.get_assets_path()
        if not self.is_valid_assets_path():
            self.logger.warning('Impossible to retrieve assets from invalid path: {}'.format(assets_path))
            return

        if not self._kitsu_is_logged:
            return

        kitsu_assets = kitsulib.all_assets_for_project(self._kitsu_id)
        asset_types = self.update_entity_types_from_kitsu(force=False)
        category_names = [asset_type.name for asset_type in asset_types]

        found_assets = list()
        for kitsu_asset in kitsu_assets:
            entity_type = self.get_entity_type_by_id(kitsu_asset.entity_type_id)
            if not entity_type:
                self.logger.warning('Entity Type {} for Asset {} is not valid! Skipping ...'.format(entity_type, kitsu_asset.name))
                continue
            asset_category = entity_type.name
            if asset_category in category_names:
                new_asset = self.create_asset(asset_data=kitsu_asset, category=asset_category)
            else:
                new_asset = self.create_asset(asset_data=kitsu_asset)
            found_assets.append(new_asset)

        return found_assets
        # return [self.ASSET_CLASS(project=self, asset_data=asset_data) for asset_data in kitsu_assets]

    @property
    def documentation_url(self):
        """
        Returns URL where Plot Twist documentation is stored
        :return: str
        """

        return self._documentation_url

    @property
    def kitsu_url(self):
        """
        Returns URL that links to Kitsu production tracker
        """

        return self._kitsu_url

    @property
    def gazu_api(self):
        """
        Returns URL that defines the Gizu API of the project
        :return: str
        """

        return self._gazu_api

    @property
    def drive_url(self):
        """
        Returns URL that links to Drive
        :return: str
        """

        return self._drive_url

    @property
    def kitsu_user(self):
        """
        Returns current Kitsu user logged
        :return: str
        """

        return self._kitsu_user

    @kitsu_user.setter
    def kitsu_user(self, user):
        """
        Sets current Kitsu user logged
        :param user: str
        """

        self._kitsu_user = user

    @property
    def kitsu_password(self):
        """
        Returns current Kitsu password logged
        :return: str
        """

        return self._kitsu_pass

    @kitsu_password.setter
    def kitsu_password(self, password):
        """
        Sets current Kitsu password logged
        :param password: str
        """

        self._kitsu_pass = password

    @property
    def kitsu_store_credentials(self):
        """
        Returns Kitsu stored credentials should be stored or not
        :return: bool
        """

        return self._kitsu_store_credentials

    @kitsu_store_credentials.setter
    def kitsu_store_credentials(self, store):
        """
        Sets whether Kitsu credentials should be stored or not
        :param store: bool
        """

        self._kitsu_store_credentials = store

    def open_webpage(self):
        """
        Opens Plot Twist official web page in browser
        """

        if not self._url:
            return

        webbrowser.open(self._url)

    def open_documentation(self):
        """
        Opens Plot Twist documentation web page in browser
        """

        if not self._documentation_url:
            return

        webbrowser.open(self._documentation_url)

    def open_kitsu(self):
        """
        Opens Plot Twist Kitsu web page in browser
        """

        if not self._kitsu_url:
            return

        webbrowser.open(self.get_kitsu_assets_url())

    def open_drive(self):
        """
        Opens Plot Twist Drive web page in browser
        """

        if not self._drive_url:
            return

        webbrowser.open(self._drive_url)

    def login_kitsu(self, email=None, password=None, store_credentials=False):
        """
        Login current project user into Kitsu with given info
        :param email: str
        :param password: str
        :param store_credentials: bool
        :return: bool
        """

        if not email:
            email = self._kitsu_user
        if not password:
            password = self._kitsu_pass
        if not store_credentials:
            store_credentials = self._kitsu_store_credentials

        if not email or not password:
            self.logger.warning('Impossible to login into Kitsu because username or password are not valid!')
            return False

        if not self.gazu_api:
            self.logger.warning('Impossible to login into Kitsu because Gazu API is not available!')
            return False

        kitsulib.set_host(self.gazu_api)
        if not kitsulib.host_is_up():
            self.logger.warning('Impossible to login into Kitsu because Gazu API is not valid: {}!'.format(self.gazu_api))
            return

        try:
            kitsulib.log_in(email, password)
            self._kitsu_is_logged = True
            self._kitsu_user_data = kitsulib.get_current_user()
            self.settings.set('kitsu_store_credentials', store_credentials)
            if store_credentials:
                self.settings.set('kitsu_email', email)
                self.settings.set('kitsu_password', password)
            return True
        except Exception:
            self._kitsu_user = None
            self._kitsu_pass = None
            self._kitsu_store_credentials = False
            self._kitsu_is_logged = False
            self._kitsu_user_data = None
            return False

    def logout_kitsu(self, remove_credentials=False):
        """
        Logout current project user from Kitsu
        """

        if not self.is_logged_in_kitsu():
            self.logger.warning('Impossible to logout from Kitsu because you are not currently logged')
            return False

        kitsulib.set_host(None)
        self._kitsu_user = None
        self._kitsu_pass = None
        self._kitsu_store_credentials = False
        self._kitsu_is_logged = False
        self._kitsu_user_data = None

        if remove_credentials:
            self.settings.set('kitsu_email', '')
            self.settings.set('kitsu_password', '')
            self.settings.set('kitsu_store_credentials', False)

        # Reinit Kitsu data
        self._init_kitsu()

        return True

    def is_logged_in_kitsu(self):
        """
        Returns whether current project user is logged into Kitsu server or not
        :return: bool
        """

        return self._kitsu_is_logged

    def open_in_kitsu(self):
        """
        Opens project Kitsu web page in user web browser
        """

        if self.kitsu_url:
            webbrowser.open(self.kitsu_url)

    def kitsu_user_data(self):
        """
        Returns current Kitsu logged user data
        :param as_dict: bool, Whether to return data as object or as dict
        :return: variant, dict or
        """

        return self._kitsu_user_data

    def update_types_from_kitsu(self, force=False):
        """
        Updates project asset types from Kitsu project
        :param force: bool, Whether to return force the update if assets types are already retrieved
        :return: list(KitsuAssetType)
        """

        if not self._kitsu_is_logged or not self._kitsu_id:
            return list()

        if self._asset_types and not force:
            return self._asset_types

        self._asset_types = kitsulib.all_asset_types_for_project(self._kitsu_id)

        return self._asset_types

    def update_entity_types_from_kitsu(self, force=False):
        """
        Updates entity types from Kitsu project
        :param force: bool, Whether to return force the update if entity types are already retrieved
        :return: list(KitsuEntityType)
        """

        if not self._kitsu_is_logged or not self._kitsu_id:
            return list()

        if self._entity_types and not force:
            return self._entity_types

        entity_types_list = kitsulib.get_project_entity_types()
        entity_types = [kitsuclasses.KitsuEntityType(entity_type) for entity_type in entity_types_list]
        self._entity_types = entity_types

        return self._entity_types

    def get_entity_type_by_id(self, entity_type_id, force_update=False):
        """
        Returns entity type name by the given project
        :param entity_type_id: str
        :param force_update: bool, Whether to force entity types sync if they are not already snced
        :return: str
        """

        if force_update or not self._entity_types:
            self.update_entity_types_from_kitsu(force=True)

        for entity_type in self._entity_types:
            if entity_type.id == entity_type_id:
                return entity_type

        return ''

    def get_kitsu_assets_url(self):
        """
        Returns URL path to Kitsu project assets
        :return: str
        """

        return self._kitsu_url + '/assets'

    def get_kitsu_shots_url(self):
        """
        Returns URL path to Kitsu project shots
        :return: str
        """

        return self._kitsu_url + '/shots'

    def get_kitsu_sequences_url(self):
        """
        Returns URL path to Kitsu project sequences
        :return: str
        """

        return self._kitsu_url + '/sequences'

    def get_kitsu_asset_types_url(self):
        """
        Returns URL path to Kitsu project asset types
        :return: str
        """

        return self._kitsu_url + '/asset-types'

    def get_kitsu_breakdown_url(self):
        """
        Returns URL path to Kitsu project breakdown
        :return: str
        """

        return self._kitsu_url + '/breakdown'

    def get_kitsu_playlists_url(self):
        """
        Returns URL path to Kitsu project playlists
        :return: str
        """

        return self._kitsu_url + '/playlists'

    def get_kitsu_team_url(self):
        """
        Returns URL path to Kitsu project team
        :return: str
        """

        return self._kitsu_url + '/team'

    def get_kitsu_news_feed_url(self):
        """
        Returns URL path to Kitsu project news-feed
        :return: str
        """

        return self._kitsu_url + '/news-feed'

    def get_kitsu_schedule_url(self):
        """
        Returns URL path to Kitsu project schedule
        :return: str
        """

        return self._kitsu_url + '/schedule'

    def _register_asset_classes(self):
        """
        Overrides base ArtellaProject _register_asset_classes function
        Internal function that can be override to register specific project asset classes
        """

        from plottwist.pipeline.tools.assetsmanager.assets import propasset, characterasset

        self.register_asset_class(propasset.PlotTwistPropAsset)
        self.register_asset_class(characterasset.PlotTwistCharacterAsset)

        super(PlotTwist, self)._register_asset_classes()

    def _register_asset_file_types(self):
        """
        Overrides base ArtellaProject _register_asset_file_types function
        Internal function that can be override to register specific project file type classes
        """

        from plottwist.core import assetfile

        self.register_asset_file_type(assetfile.TexturesAssetFile)
        self.register_asset_file_type(assetfile.ModelAssetFile)
        self.register_asset_file_type(assetfile.ShadingAssetFile)
        self.register_asset_file_type(assetfile.RigAssetFile)
        self.register_asset_file_type(assetfile.GroomAssetFile)
        self.register_asset_file_type(assetfile.AlembicAssetFile)

        super(PlotTwist, self)._register_asset_file_types()

    def _init_kitsu(self):
        """
        Initializes Kitsu data
        """

        self._kitsu_user = self.settings.get('kitsu_email') if self.settings.has_setting('kitsu_email') else None
        self._kitsu_pass = self.settings.get('kitsu_password') if self.settings.has_setting('kitsu_password') else None
        self._kitsu_store_credentials = self.settings.get('kitsu_store_credentials') if self.settings.has_setting('kitsu_store_credentials') else False